@@ -3,25 +3,23 @@ import typing
 import pandas as pd
 
 from tabml.feature_manager import BaseFeatureManager
+from tabml.schemas import pipeline_config
 
 
 class BaseDataLoader:
     """Base class for DataLoader.
 
-    Support extract relevant dataset based on feature manager.
+    Support extract relevant datasets based on feature manager.
     """
 
-    def __init__(self, config):
-        self.config = config
-        self.label_col = self.config.data_loader.label_col
-        self.features = list(self.config.data_loader.features_to_model)
+    def __init__(self, params=pipeline_config.DataLoader()):
+        self.label_col = params.label_col
+        self.features = params.features_to_model
         self.features_and_label = self.features + [self.label_col]
-        self.feature_manager = self._get_feature_manager()
+        self.feature_manager = BaseFeatureManager(params.feature_config_path)
         self.feature_manager.load_dataframe()
-
-    def _get_feature_manager(self):
-        fm_config_path = self.config.data_loader.feature_manager_config_path
-        return BaseFeatureManager(fm_config_path)
+        self.train_filters = params.train_filters
+        self.validation_filters = params.validation_filters
 
     def _extract_data_and_label(
         self, filters: typing.List[str]
@@ -35,8 +33,8 @@ class BaseDataLoader:
 
     def get_train_data_and_label(self) -> typing.Tuple[pd.DataFrame, pd.Series]:
         """Gets training data and label."""
-        return self._extract_data_and_label(self.config.data_loader.train_filters)
+        return self._extract_data_and_label(self.train_filters)
 
     def get_val_data_and_label(self):
         """Gets validation dataset."""
-        return self._extract_data_and_label(self.config.data_loader.validation_filters)
+        return self._extract_data_and_label(self.validation_filters)
