@@ -2,29 +2,37 @@ import copy
 from typing import Dict, List, Union
 
 from tabml.config_helpers import parse_feature_config
-from tabml.schemas.feature_config import DType
+from tabml.schemas.feature_config import BaseFeature, DType, TransformingFeature
 from tabml.utils.utils import check_uniqueness
 
 
-class _Feature:
+class FeatureMetadata:
     def __init__(
         self, index: int, dtype: DType, dependencies: Union[List, None] = None
     ):
         self.index = index
         self.dependents: List[str] = []
         self.dtype = dtype
-        if dependencies is None:
-            self.dependencies = []
-        else:
-            self.dependencies = dependencies
+        self.dependencies = dependencies or []
+
+    @classmethod
+    def from_base_feature(cls, feature: BaseFeature):
+        # All base features are considered to have index 0.
+        return cls(index=0, dtype=feature.dtype)
+
+    @classmethod
+    def from_transforming_feature(cls, feature: TransformingFeature):
+        return cls(
+            index=feature.index, dtype=feature.dtype, dependencies=feature.dependencies
+        )
 
 
 class FeatureConfigHelper:
     """A config helper class for feature manager.
 
     Attributes:
-        _config:
-            A protobuf object parsed from a pb text file.
+        config:
+            A feature_config object parsed from a yaml file.
         raw_data_dir:
             A string of directory to raw data.
         dataset_name:
@@ -41,17 +49,17 @@ class FeatureConfigHelper:
     """
 
     def __init__(self, config_path: str):
-        self._config = parse_feature_config(config_path)
-        self.raw_data_dir = self._config.raw_data_dir
-        self.dataset_name = self._config.dataset_name
-        self.base_features = [feature.name for feature in self._config.base_features]
+        self.config = parse_feature_config(config_path)
+        self.raw_data_dir = self.config.raw_data_dir
+        self.dataset_name = self.config.dataset_name
+        self.base_features = [feature.name for feature in self.config.base_features]
         self.transforming_features = [
             transforming_feature.name
-            for transforming_feature in self._config.transforming_features
+            for transforming_feature in self.config.transforming_features
         ]
         self.all_features = self.base_features + self.transforming_features
         self._validate()
-        self.feature_metadata: Dict[str, _Feature] = {}
+        self.feature_metadata: Dict[str, FeatureMetadata] = {}
         self._build_feature_metadata()
 
     def _validate(self):
@@ -62,9 +70,8 @@ class FeatureConfigHelper:
     def _validate_indexes(self):
         """Checks if indexes are positive and monotonically increasing."""
         indexes = [
-            # TODO: why we need int() here? Does yaml parse this incorrectly?
-            int(transforming_feature.index)
-            for transforming_feature in self._config.transforming_features
+            transforming_feature.index
+            for transforming_feature in self.config.transforming_features
         ]
         if not (
             indexes[0] > 0
@@ -81,10 +88,10 @@ class FeatureConfigHelper:
             )
 
     def _validate_dependency(self):
-        """Checks if all dependencies of a transforming feature exists."""
-        # initialize from base_features then gradually adding transforming_feature
+        """Checks if all dependencies of a transforming feature exist."""
+        # Start from base_features then gradually add transforming_features.
         features_so_far = self.base_features.copy()
-        for feature in self._config.transforming_features:
+        for feature in self.config.transforming_features:
             for dependency in feature.dependencies:
                 assert (
                     dependency in features_so_far
@@ -94,30 +101,18 @@ class FeatureConfigHelper:
             features_so_far.append(feature.name)
 
     def _build_feature_metadata(self):
-        for feature in self._config.base_features:
-            # all base features are considered to have index 0
-            self.feature_metadata[feature.name] = _Feature(index=0, dtype=feature.dtype)
-
-        for feature in self._config.transforming_features:
-            self.feature_metadata[feature.name] = _Feature(
-                index=feature.index,
-                dtype=feature.dtype,
-                dependencies=feature.dependencies,
+        for feature in self.config.base_features:
+            self.feature_metadata[feature.name] = FeatureMetadata.from_base_feature(
+                feature
             )
+
+        for feature in self.config.transforming_features:
+            self.feature_metadata[
+                feature.name
+            ] = FeatureMetadata.from_transforming_feature(feature)
             for dependency in feature.dependencies:
                 self.feature_metadata[dependency].dependents.append(feature.name)
 
-    def sort_features(self, feature_names: List[str]) -> List[str]:
-        # TODO: fix parsing index as int in yaml
-        return sorted(feature_names, key=lambda x: int(self.feature_metadata[x].index))
-
-    def find_dependencies(self, feature_name: str) -> List[str]:
-        # NOTE: repeated fields in proto are NOT parsed to python lists but
-        # <class 'google.protobuf.pyext._message.RepeatedScalarContainer'>.
-        # pandas 1.1.4 accepts the above type as an array of column names but
-        # pandas 1.3.1 does not!
-        return list(self.feature_metadata[feature_name].dependencies)
-
     def get_dependencies_recursively(self, features: List[str]) -> List[str]:
         """Gets all dependencies of a list of features recursively.
 
@@ -130,22 +125,28 @@ class FeatureConfigHelper:
             if feature in seen:
                 continue
             seen.append(feature)
-            queue.extend(self.find_dependencies(feature))
+            queue.extend(self.get_direct_dependencies(feature))
         return self.sort_features(seen)
 
-    def find_dependents(self, feature_name: str) -> List[str]:
-        """Finds all features that are directly/indirectly dependent on a feature.
+    def get_direct_dependencies(self, feature_name: str) -> List[str]:
+        return self.feature_metadata[feature_name].dependencies
+
+    def sort_features(self, feature_names: List[str]) -> List[str]:
+        return sorted(feature_names, key=lambda x: self.feature_metadata[x].index)
+
+    def get_dependents_recursively(self, feature_name: str) -> List[str]:
+        """Finds all features that are directly/indirectly dependents of a feature.
 
         This is necessary when we want to update one feature. All of its dependents also
         need to be updated. The list of returning features is required to be in the
-        order determined by their indexes in the proto config.
+        order determined by their indexes in the config.
 
         Notes:
             * If "b" is a dependency of "a" then "a" is a dependent of "b".
             * If "a" is a dependent of "b" and "b" is a dependent of "c" then "a" is a
               dependent of "c".
         """
-        # BFS to find all dependents
+        # BFS to find all dependents.
         dependents: List[str] = []
         queue = self.feature_metadata[feature_name].dependents
         while queue:
@@ -156,26 +157,11 @@ class FeatureConfigHelper:
             queue.extend(self.feature_metadata[feature].dependents)
         return self.sort_features(dependents)
 
-    def append_dependents(self, feature_names: List[str]) -> List[str]:
-        """Finds all dependents of a list of features then appends them to the list.
-
-        This will be used when multiple features need to be updated or remove. For the
-        first case, all dependents also need to be updated. For the second case, it's
-        required that there is no dependent in the remaining features.
-
-        NOTE: the results should also contain the input feature_names.
-        """
-        all_features = feature_names
-        for feature_name in feature_names:
-            all_features.extend(self.find_dependents(feature_name))
-
-        return self.sort_features(list(set(all_features)))
-
     def extract_config(self, selected_features: List[str]):
         """Creates a minimum valid config that contains all selected_features.
 
-        Returns a protobuf with only a subset of transforming features and all of their
-        dependencies.
+        Returns a feature_config object with only a subset of transforming features and
+        all of their dependencies.
 
         NOTE: all base features will be in the extracted config; they were created in
         the raw data loading step.
@@ -183,25 +169,33 @@ class FeatureConfigHelper:
         Args:
             selected_features: a list of selected features.
 
+        """
+        self._validate_features(selected_features)
+
+        new_config = copy.deepcopy(self.config)
+        required_features = self.get_dependencies_recursively(
+            features=selected_features
+        )
+        minimum_transforming_features = [
+            transforming_feature
+            for transforming_feature in self.config.transforming_features
+            if transforming_feature.name in required_features
+        ]
+        new_config.transforming_features = minimum_transforming_features
+
+        return new_config
+
+    def _validate_features(self, features: List[str]):
+        """Checks if all features are in the config.
+
         Raises:
             ValueError if selected_features contains an unknown features (not in the
             original config).
         """
         invalid_features = [
-            feature for feature in selected_features if feature not in self.all_features
+            feature for feature in features if feature not in self.all_features
         ]
         if invalid_features:
             raise ValueError(
                 f"Features {invalid_features} are not in the original config."
             )
-        all_relevant_features = self.get_dependencies_recursively(
-            features=selected_features
-        )
-        new_config = copy.deepcopy(self._config)
-        new_config.transforming_features = [
-            transforming_feature
-            for transforming_feature in self._config.transforming_features
-            if transforming_feature.name in all_relevant_features
-        ]
-
-        return new_config
